package de.intevation.flys.artifacts.model;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;

import java.io.Serializable;

import de.intevation.artifacts.CallMeta;

import de.intevation.flys.artifacts.resources.Resources;

public class Calculation
implements   Serializable
{
    public static class Problem
    implements          Serializable
    {
        protected Double    km;
        protected String    msg;
        protected Object [] args;

        public Problem() {
        }

        public Problem(String msg) {
            this.msg = msg;
        }

        public Problem(String msg, Object [] args) {
            this.msg  = msg;
            this.args = args;
        }

        public Problem(double km, String msg) {
            this.km  = km;
            this.msg = msg;
        }

        public Problem(double km, String msg, Object [] args) {
            this.km   = km;
            this.msg  = msg;
            this.args = args;
        }

        public Element toXML(Document document, CallMeta meta) {
            Element problem = document.createElement("problem");
            if (km != null) {
                problem.setAttribute("km", String.valueOf(km));
            }
            String text = args != null
                ? Resources.getMsg(meta, msg, msg, args)
                : Resources.getMsg(meta, msg, msg);
            problem.setTextContent(text);
            return problem;
        }

        @Override
        public boolean equals(Object other) {
            if (!(other instanceof Problem)) {
                return false;
            }
            Problem o = (Problem)other;
            return !(!msg.equals(o.msg)
                || (km == null && o.km != null)
                || (km != null && o.km == null)
                || (km != null && !km.equals(o.km))
                || !Arrays.equals(args, o.args));
        }
    } // class Problem

    protected List<Problem> problems;

    public Calculation() {
    }

    public Calculation(String msg) {
        addProblem(msg);
    }

    protected List<Problem> checkProblems() {
        if (problems == null) {
            problems = new ArrayList<Problem>();
        }
        return problems;
    }

    public void addProblem(Problem problem) {
        List<Problem> problems = checkProblems();
        if (!problems.contains(problem)) {
            problems.add(problem);
        }
    }

    public void addProblem(String msg) {
        addProblem(new Problem(msg));
    }

    public void addProblem(String msg, Object ... args) {
        addProblem(new Problem(msg, args));
    }

    public void addProblem(double km, String msg) {
        addProblem(new Problem(km, msg));
    }

    public void addProblem(double km, String msg, Object ... args) {
        addProblem(new Problem(km, msg, args));
    }

    public boolean hasProblems() {
        return problems != null && !problems.isEmpty();
    }

    public int numProblems() {
        return problems != null ? problems.size() : 0;
    }

    public List<Problem> getProblems() {
        return problems;
    }

    public void toXML(Document document, CallMeta meta) {

        Element root = document.createElement("problems");

        if (hasProblems()) {
            for (Problem problem: problems) {
                root.appendChild(problem.toXML(document, meta));
            }
        }

        document.appendChild(root);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
